==> rstudio-0.97.336.tar.gz: rstudio-0.97.336/src/cpp/server/auth/ServerValidateUser.cpp <==
/*
 * ServerValidateUser.cpp
 *
 * Copyright (C) 2009-12 by RStudio, Inc.
 *
 * Unless you have received this program directly from RStudio pursuant
 * to the terms of a commercial license agreement with RStudio, then
 * this program is licensed to you under the terms of version 3 of the
 * GNU Affero General Public License. This program is distributed WITHOUT
 * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
 * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
 *
 */

#include <server/auth/ServerValidateUser.hpp>

#include <core/Error.hpp>
#include <core/Log.hpp>
#include <core/StringUtils.hpp>

#include <core/system/PosixSystem.hpp>
#include <core/system/PosixUser.hpp>

#include <server/ServerOptions.hpp>

using namespace core;

namespace server {
namespace auth {

bool validateUser(const std::string& username)
{
   // short circuit if we aren't validating users
   if (!server::options().authValidateUsers())
      return true;
   
   // get the user
   core::system::user::User user;
   Error error = userFromUsername(username, &user);
   if (error)
   {
      // log the error only if it is unexpected
      if (!core::system::isUserNotFoundError(error))
         LOG_ERROR(error);

      // not found either due to non-existence or an unexpected error
      return false;
   }

   // validate user if necessary
   std::string requiredGroup = server::options().authRequiredUserGroup();
   if (!requiredGroup.empty())
   {    
      // see if they are a member of the "rstudio_users" group
      bool belongsToGroup ;
      error = core::system::userBelongsToGroup(username,
                                               requiredGroup,
                                               &belongsToGroup);
      if (error)
      {
         // log and return false
         LOG_ERROR(error);
         return false;
      }
      else
      {
         // return belongs status
         return belongsToGroup;
      }
   }
   else
   {
      // not validating (running in some type of dev mode where we
      // don't have a system account for every login)
      return true;
   }
}

} // namespace auth
} // namespace server

==> rstudio-0.98.953.tar.gz: rstudio-0.98.953/src/cpp/server/auth/ServerValidateUser.cpp <==
/*
 * ServerValidateUser.cpp
 *
 * Copyright (C) 2009-12 by RStudio, Inc.
 *
 * Unless you have received this program directly from RStudio pursuant
 * to the terms of a commercial license agreement with RStudio, then
 * this program is licensed to you under the terms of version 3 of the
 * GNU Affero General Public License. This program is distributed WITHOUT
 * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
 * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
 *
 */

#include <server/auth/ServerValidateUser.hpp>

#include <core/Error.hpp>
#include <core/Log.hpp>
#include <core/StringUtils.hpp>

#include <core/system/PosixSystem.hpp>
#include <core/system/PosixUser.hpp>

#include <server/ServerOptions.hpp>

using namespace core;

namespace server {
namespace auth {

bool validateUser(const std::string& username,
                  const std::string& requiredGroup,
                  bool groupFailureWarning)
{
   // short circuit if we aren't validating users
   if (!server::options().authValidateUsers())
      return true;
   
   // get the user
   core::system::user::User user;
   Error error = userFromUsername(username, &user);
   if (error)
   {
      // log the error only if it is unexpected
      if (!core::system::isUserNotFoundError(error))
         LOG_ERROR(error);

      // not found either due to non-existence or an unexpected error
      return false;
   }

   // validate user if necessary
   if (!requiredGroup.empty())
   {    
      // see if they are a member of the required group
      bool belongsToGroup ;
      error = core::system::userBelongsToGroup(user,
                                               requiredGroup,
                                               &belongsToGroup);
      if (error)
      {
         // log and return false
         LOG_ERROR(error);
         return false;
      }
      else
      {
         // log a warning whenever a user doesn't belong to a required group
         if (!belongsToGroup && groupFailureWarning)
         {
            LOG_WARNING_MESSAGE(
             "User " + username + " could not be authenticated because they "
             "do not belong to the required group (" + requiredGroup + ")");
         }

         // return belongs status
         return belongsToGroup;
      }
   }
   else
   {
      // not validating (running in some type of dev mode where we
      // don't have a system account for every login)
      return true;
   }
}

} // namespace auth
} // namespace server
